# Generated by Django 4.1.9 on 2023-06-21 16:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0016_alter_folder_index_together_remove_folder_level_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='image',
            name='_transparent',
            field=models.BooleanField(default=False),
        ),
    ]
