# Generated by Django 3.0.8 on 2020-10-28 16:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0012_file_mime_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='image',
            name='_height',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='image',
            name='_width',
            field=models.FloatField(blank=True, null=True),
        ),
    ]
